using System;
using System.Globalization;
using System.IO;
using System.Web.Compilation;
using System.Web.Mvc;

namespace MvcLocalization
{
    public class LocalizationWebFormView : WebFormView
    {
        public LocalizationWebFormView(string viewPath) : base(viewPath)
        {
        }

        public LocalizationWebFormView(string viewPath, string masterPath) : base(viewPath, masterPath)
        {
        }

        public override void Render(ViewContext viewContext, TextWriter writer)
        {
            if (viewContext == null)
            {
                throw new ArgumentNullException("viewContext");
            }

            object viewInstance = BuildManager.CreateInstanceFromVirtualPath(ViewPath, typeof(object));
            if (viewInstance == null)
            {
                throw new InvalidOperationException(
                    String.Format(
                        CultureInfo.CurrentUICulture,
                        "View cannot be created",
                        ViewPath));
            }

            ViewPage viewPage = viewInstance as ViewPage;
            if (viewPage != null)
            {
                RenderViewPage(viewContext, viewPage);
                return;
            }

            ViewUserControl viewUserControl = viewInstance as ViewUserControl;
            if (viewUserControl != null)
            {
                RenderViewUserControl(viewContext, viewUserControl);
                return;
            }

            throw new InvalidOperationException(
                string.Format(
                    CultureInfo.CurrentUICulture,
                    "Wrong View Base",
                    ViewPath));
        }

        private void RenderViewPage(ViewContext context, ViewPage page)
        {
            if (!string.IsNullOrEmpty(MasterPath))
            {
                page.MasterLocation = MasterPath;
            }

            page.ViewData = context.ViewData;
            var localizedPage = page as ILocalizedView;
            if (localizedPage != null)
                localizedPage.Resource = new WebFormResourceHelper(ViewPath);
            page.RenderView(context);
        }

        private void RenderViewUserControl(ViewContext context, ViewUserControl control)
        {
            if (!String.IsNullOrEmpty(MasterPath))
            {
                throw new InvalidOperationException("User control cannot have a master");
            }

            control.ViewData = context.ViewData;
            var localizedControl = control as ILocalizedView;
            if (localizedControl != null)
                localizedControl.Resource = new WebFormResourceHelper(ViewPath);
            control.RenderView(context);
        }
    }
}